﻿USING System
USING System.Collections.Generic
USING System.Linq
USING System.Text
USING System.Diagnostics

BEGIN NAMESPACE LinqSample

	FUNCTION Start() AS VOID
		FirstSample()
		SecondSample()
		ThirdSample()
		
	FUNCTION FirstSample() AS VOID    
		// The Three Parts of a LINQ Query:
		//  1. Data source.
		LOCAL numbers AS INT[]
		numbers = <INT>{ 0, 1, 2, 3, 4, 5, 6 }
		
		// 2. Query creation.
		// numQuery is an IEnumerable<int>
		VAR numQuery = ;
		FROM num IN numbers ;
		WHERE (num % 2) == 0 ;
		SELECT num
		
		// 3. Query execution.
		FOREACH num AS INT IN numQuery
			Console.Write( num.ToString() + " ")
		NEXT
		
		Console.WriteLine("Press any key to continue...")
		Console.ReadKey()
		
		
	FUNCTION SecondSample() AS VOID    
		//
		LOCAL numbers AS INT[]
		numbers = <INT>{ 1, 2, 3, 4, 5, 6, 7, 8 }
		
		// Query expression.
		VAR elements = FROM element IN numbers ;
		ORDERBY element DESCENDING ;
		WHERE element > 2 ;
		SELECT element
		// Enumerate.
		FOREACH element AS INT IN elements
			Console.Write( element.ToString() + " ")
		NEXT
		
		Console.WriteLine("Press any key to continue...")
		Console.ReadKey()
		
		
	FUNCTION ThirdSample() AS VOID
		LOCAL repeat = 1000000 AS INT
		LOCAL count AS INT
		LOCAL numbers AS INT[]
		numbers = <INT>{ 10, 0, 1, 1, 20, 300, 400, 4 }
		//
		// Version 1: use LINQ.
		VAR s1 = Stopwatch.StartNew()
		FOR VAR I = 1 TO repeat
			count = CountLinq(numbers)
		NEXT
		s1.Stop()
		
		// Version 2: use for-loop.
		VAR s2 = Stopwatch.StartNew()
		FOR VAR I = 1 TO repeat
			count = CountFor(numbers)
		NEXT
		s2.Stop()
		//
		Console.WriteLine( "Linq version : " + ((double)(s1.Elapsed.TotalMilliseconds * 1000000) / repeat).ToString("0.00 ns"))
		Console.WriteLine( "For-Loop version : " + ((double)(s2.Elapsed.TotalMilliseconds * 1000000) / repeat).ToString("0.00 ns"))
		//
		Console.WriteLine("Press any key to continue...")
		Console.ReadKey()
		
		
		
	FUNCTION CountLinq( values AS INT[] ) AS INT
		// Count values greater than or equal to 10 with LINQ.
		RETURN (FROM x IN values WHERE x >= 10 SELECT x).Count()
		
	FUNCTION CountFor( values AS INT[] ) AS INT
		// Count values greater than or equal to 10 with a loop.
		LOCAL count = 0 AS INT
		FOR VAR i = 1 TO values.Length
			IF (values[i] >= 10)
				count++
			ENDIF
		NEXT
		RETURN count
		
END NAMESPACE
